/* $Id: CodeGen.hpp,v 1.11 2004/11/21 18:52:51 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Defines the codegenerator of the hasi project.
 */
#ifndef HASI_CODEGEN_HPP
#define HASI_CODEGEN_HPP

// system includes:
#include <queue>
#include <string>

// program includes:
#include "SymbolTable.hpp"
#include "Lexer.hpp"
#include "Error.hpp"

namespace CodeGenerator {

  /* The struct that holds information for each */
  /* mnemonic and each meta instruction: */
  typedef struct {
    char *mnemonic;
    int   opcode;
    int   modes;
  } OPCODE_t;

  /* The code-generator class */
  class CodeGen
  {
  private:
    Sourcehandler::SourceHandler &SH;
    int IP;
    std::queue<int> opcode_lengths;

    // Private member functions:
    void emit_code(const int code, const int data) const;
    void emit_data(const int data) const;
    void emit_string(const std::string &s) const;
    int atoi(const std::string &num_string) const;
  public:
    CodeGen(Sourcehandler::SourceHandler &SH) : SH(SH) { IP = 0; }
    virtual ~CodeGen();

    // Member functions:
    virtual const OPCODE_t *lookupOpcode(const std::string &opcode);
    virtual bool lookupMeta(const std::string &meta);
    virtual bool doMeta(const std::string &label, const std::string &meta,
                        const std::string &arg, Lexer::ARG_TYPE_t type,
                        SymbolTable::SymbolTable &, const int pass,
                        const bool negative)
      throw (Error::syntax_error, Error::internal_error);
    virtual bool generate(const std::string &op, const std::string &arg,
                          const Lexer::ARG_TYPE_t mode,
                          SymbolTable::SymbolTable &, const int pass,
                          const bool lastpass, const bool negative)
      throw (Error::syntax_error, Error::internal_error);
    virtual int getIP() const { return IP; }
    virtual void rewind() { IP = 0; }
  };

}

#endif
