/* $Id: Error.hpp,v 1.5 2003/12/12 09:47:26 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Definition of exceptions thrown at different error events.
 */
#ifndef HASI_AS_ERROR_HPP
#define HASI_AS_ERROR_HPP

// system includes:
#include <string>
#include <exception>

namespace Error {
  
  /*
   * Base class for our error messages 
   */
  class error : public std::exception
  {
  public:
    error() { };
    virtual ~error() throw ();

    virtual void emit() = 0;
  };

  /*
   * A syntax error
   */
  class syntax_error : public error
  {
  private:
    std::string message;
    int         lineno;
  public:
    syntax_error(const std::string &msg, const int lineno);
    virtual ~syntax_error() throw ();

    virtual void emit();
  };

  /*
   * An internal error, something went completely wrong if this occurs 
   */
  class internal_error : public error
  {
  private:
    int lineno;
  public:
    internal_error(const int lineno);
    virtual ~internal_error() throw ();

    virtual void emit();
  };

  /*
   * Base class for file I/O errors
   */
  class file_error : public error
  {
  protected:
    std::string filename;
  public:
    file_error(const std::string &filename);
    virtual ~file_error() throw ();

    virtual void emit() = 0;
  };

  /*
   * Error if a file could not be opened 
   */
  class file_open_error : public file_error
  {
  public:
    file_open_error(const std::string &filename);
    virtual ~file_open_error() throw ();
    
    virtual void emit();
  };

  /*
   * Error if a file could not be read 
   */
  class file_read_error : public file_error
  {
  public:
    file_read_error(const std::string &filename);
    virtual ~file_read_error() throw ();
    
    virtual void emit();
  };

  /*
   * Error if a write to a file did not finish successful
   */
  class file_write_error : public file_error
  {
  public:
    file_write_error(const std::string &filename);
    virtual ~file_write_error() throw ();
    
    virtual void emit();
  };

}

#endif
