/* $Id: Parser.hpp,v 1.7 2004/11/21 18:52:51 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Definition of the parser class.
 */
#ifndef HASI_PARSER_HPP
#define HASI_PARSER_HPP

// program includes:
#include "SourceHandler.hpp"
#include "Lexer.hpp"
#include "CodeGen.hpp"
#include "SymbolTable.hpp"
#include "Error.hpp"

namespace Parser {

  /*
   * class Parser
   */
  class Parser
  {
  private:
    Sourcehandler::SourceHandler &SH;
    Lexer::LexicalAnalyzer &LA;
    CodeGenerator::CodeGen &GEN;
    SymbolTable::SymbolTable &ST;
  public:
    // Con- / Destructor:
    Parser(Sourcehandler::SourceHandler &, Lexer::LexicalAnalyzer &,
           CodeGenerator::CodeGen &, SymbolTable::SymbolTable &);
    ~Parser() { }

    // parse is for now the only function
    bool parse(const int pass, const bool lastpass)
      throw (Error::syntax_error, Error::internal_error);
    // rewind all open files and start parsing from the beginning
    void rewind();
  };

}

#endif
