/* $Id: SourceHandler.hpp,v 1.7 2003/12/12 09:47:26 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Definition of the source handler class. 
 */
#ifndef HASI_SOURCEHANDLER_HPP
#define HASI_SOURCEHANDLER_HPP

// system includes:
#include <fstream>
#include <list>
#include <string>

// program includes:
#include "Error.hpp"

namespace Sourcehandler {

  class SourceHandler
  {
  private:
    // the filehandles:
    std::ifstream *src;            // Source file
    std::ofstream *lst;            // List file
    std::ofstream *obj;            // Object file
    std::ostream  *obj_output;

    bool obj_is_stdout;

    // sourcefile handling:
    int pos;                       // Position in the line
    int linelength;                // Length of the last read line
    int linenr;                    // Actual line number
    std::string line;              // The actual line buffer
    int ch;                        // Last read character

    // the filenames:
    std::string srcname, lstname, objname;  // The filenames

    // the list and objfile representations:
    std::list<std::string> objfile, listfile;

    // listfile_ip holds the actual ip reported to the sourcehandler
    // because there is no direct way to call the codegenerator
    unsigned int listfile_ip;

public:
    // Constructors and Destructor:
    SourceHandler(const std::string &sourcename, const std::string &listname,
                  const std::string &objname)
      throw(Error::file_open_error);
    ~SourceHandler(); 

    // Multiple reads of the source file are to be done
    void rewind();

    // Operations for the source file:
    bool beginline() const { return (pos == 1); };
    bool endline() const { return (pos == linelength); };
    int nextchar() throw(Error::file_read_error);
    int getchar() const { return ch; }

    // Operations for the list file:
    void list_write_hex(unsigned int, int n);
    void list_write_ascii(const std::string &s, int n);
    void list_write() throw(Error::file_write_error);
    void list_set_ip(const unsigned int newip) { listfile_ip = newip; }

    void obj_write_hex(unsigned int, int n);
    void obj_write_ascii(const std::string &s, int n);
    void obj_write() throw(Error::file_write_error);

    // Some functions for error handling:
    int getLineNr() const { return linenr - 1; }
    const std::string &getLine() const { return line; }
  };

}

#endif
