/* $Id: SymbolTable.cpp,v 1.5 2003/12/12 09:47:26 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * Implementation of the symbol table.
 */

// system includes:
#ifdef DEBUG
// this is only needed if we want to dump the symbol table
#include <iostream>
#include <iomanip>
#endif

// program includes:
#include "SymbolTable.hpp"

namespace SymbolTable {

  typedef std::map<std::string, Symbol>::const_iterator CI;

  // const Symbol *SymbolTable::lookup(const std::string &) const:
  // Searches for a symbol with the specified name in the table and
  // returns a pointer to the element.
  const Symbol *SymbolTable::lookup(const std::string &name) const
  {
    CI it = table.find(name);
    
    if (it == table.end()) return NULL;
    return &it->second;
  }

  // bool SymbolTable::change(const Symbol &):
  // Changes or inserts a symbol in the symbol table. true is returned in
  // case that an existing element has been changed. false if a new entry
  // is made.
  bool SymbolTable::change(const Symbol &sym)
  {
    CI it = table.find(sym.getName());
    // if it is not in the map we will create one automagically
    // and return false
    bool return_val = (it != table.end()) ? true : false;
    table[sym.getName()] = sym;
    return return_val;
  }

#ifdef DEBUG
  // void SymbolTable::dump() const
  // Dumps the content of the symoboltable to stderr.
  void SymbolTable::dump() const 
  {
    std::cerr << table.size() << " labels defined." << std::endl;
    CI it = table.begin();
    while (it != table.end()) {
      std::cerr.fill('0');
      std::cerr << "Label: " << it->first << " = $"
                << std::setw(4) << std::hex << it->second.getValue()
                << std::endl;
      it++;
    }
  }
#endif

}

