/* $Id: MainForm.hpp,v 1.14 2003/12/12 09:47:26 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,

 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * The main window.
 */
#ifndef HASI_MAINFORM_HPP
#define HASI_MAINFORM_HPP

// this has to be included before the QT includes occur. I don't know why,
// but if it is included after the QT includes gcc reports an error...
#include "Error.hpp"

// system includes:
#include <qmainwindow.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qtextedit.h>
#include <qwidgetstack.h>

// program includes:
#include "QMemoryTable.hpp"
#include "QMemoryPane.hpp"
#include "QDisassemblerPane.hpp"
#include "CPU.hpp"
#include "QDebugPane.hpp"
#include "Options.hpp"
#include "QHelpWindow.hpp"
#include "QStackTable.hpp"

namespace GUI {

  /*
   * class MainForm
   */
  class MainForm : public QMainWindow
  {
    Q_OBJECT

  private:
    Interpreter::CPU &cpu;
    QWidgetStack *mainWidgetStack;
    QWidget      *editorPage;
    QWidget      *debuggerPage;
    QTextEdit    *sourceTextEdit;
    QMemoryPane  *memoryTable;
    QDisassemblerPane *disassTable;
    QStackTable  *stackTable;
    QDebugPane   *debugTable;
    QMenuBar     *menuBar;
    QPopupMenu   *fileMenu;
    QPopupMenu   *editMenu;
    QPopupMenu   *assemblerMenu;
    QPopupMenu   *debugMenu;
    QPopupMenu   *helpMenu;
    QAction      *fileNewAction;
    QAction      *fileOpenAction;
    QAction      *fileSaveAction;
    QAction      *fileSaveAsAction;
    //QAction      *filePrintAction;
    QAction      *fileExitAction;
    QAction      *editUndoAction;
    QAction      *editRedoAction;
    QAction      *editCutAction;
    QAction      *editCopyAction;
    QAction      *editPasteAction;
    QAction      *editFindAction;
    QAction      *assemblerSwitchDebugAction;
    QAction      *assemblerAssembleAction;
    QAction      *assemblerOptionsAction;
    QAction      *debugSwitchEditAction;
    QAction      *debugStepAction;
    QAction      *debugNextAction;
    QAction      *debugRunAction;
    QAction      *helpContentsAction;
    QAction      *helpIndexAction;
    QAction      *helpAboutAction;
    QLabel       *cursorPositionLabel;

    // set to true if the document has changed
    bool documentChanged;
    std::string documentName;

    Options *opts;
    QHelpWindow *help;

  public:
    MainForm(Interpreter::CPU &cpu, QWidget *parent = NULL,
             const char *name = NULL, 
             WFlags fl = WType_TopLevel);
    ~MainForm();

  public slots:
    virtual void fileNew();
    virtual void fileOpen();
    virtual void fileSave();
    virtual void fileSaveAs();
    //virtual void filePrint();
    virtual void fileExit();
    virtual void editUndo();
    virtual void editRedo();
    virtual void editCut();
    virtual void editCopy();
    virtual void editPaste();
    virtual void editFind();
    virtual void helpIndex();
    virtual void helpContents();
    virtual void helpAbout();
    virtual void assemblerSwitchDebug();
    virtual void assemblerAssemble();
    virtual void assemblerOptions();
    virtual void debugSwitchEdit();

    virtual void textChanged();
    virtual void cursorPositionChanged(int, int);

  protected:
    QVBoxLayout *mainFormLayout;
    QVBoxLayout * editorPageLayout;

    bool Debugger;

    void closeEvent(QCloseEvent *);

  protected slots:
    virtual void languageChange();
    void Run(bool);

    // for the text edit field
    void copyStatus(bool available);
    void undoStatus(bool available);
    void redoStatus(bool available);
  };

}

#endif // HASI_MAINFORM_HPP
