/* $Id: Options.cpp,v 1.6 2003/12/12 09:47:26 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Definition of a class holding the options to invoke the assembler.
 * The class is derived from QDialog and can show a dialog if necessary.
 */

// system includes:
#include <qlayout.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qfiledialog.h>

// program includes:
#include "Options.hpp"

namespace GUI {

#define CONFIG_FILE "/.hasirc"

  // Options::Options(QWidget *parent, const char *name)
  // Sets up the layout and functionality of the Options dialog.
  Options::Options(QWidget *parent, const char *name)
    : QDialog(parent, name), asPath("."), helpPath("./help"),
      listfile(false), passes(100)
  {
    if (! name)
      setName("Options");

    OptionsLayout = new QVBoxLayout(this, 11, 6, "OptionsLayout"); 

    groupBox1 = new QGroupBox(this, "groupBox1");
    groupBox1->setColumnLayout(0, Qt::Vertical);
    groupBox1->layout()->setSpacing(6);
    groupBox1->layout()->setMargin(11);
    groupBox1Layout = new QVBoxLayout(groupBox1->layout());
    groupBox1Layout->setAlignment(Qt::AlignTop);

    layout1 = new QHBoxLayout(0, 0, 6, "layout1"); 

    textLabel1 = new QLabel(groupBox1, "textLabel1");
    layout1->addWidget(textLabel1);

    pathnameLineEdit = new QLineEdit(groupBox1, "pathnameLineEdit");
    layout1->addWidget(pathnameLineEdit);

    fileBrowserButton = new QPushButton(groupBox1, "fileBrowserButton");
    layout1->addWidget(fileBrowserButton);
    groupBox1Layout->addLayout(layout1);

    layout4 = new QHBoxLayout(0, 0, 6, "layout4"); 

    textLabel3 = new QLabel(groupBox1, "textLabel3");
    layout4->addWidget(textLabel3);

    pathnameHelpLineEdit = new QLineEdit(groupBox1, "pathnameHelpLineEdit");
    layout4->addWidget(pathnameHelpLineEdit);

    fileBrowserHelpButton = new QPushButton(groupBox1, "fileBrowserButton2");
    layout4->addWidget(fileBrowserHelpButton);
    groupBox1Layout->addLayout(layout4);

    layout2 = new QHBoxLayout(0, 0, 6, "layout2"); 

    listfileCheckBox = new QCheckBox(groupBox1, "listfileCheckBox");
    layout2->addWidget(listfileCheckBox);
    QSpacerItem *spacer = new QSpacerItem(331, 20, QSizePolicy::Expanding,
                                          QSizePolicy::Minimum);
    layout2->addItem(spacer);
    groupBox1Layout->addLayout(layout2);

    layout3 = new QHBoxLayout(0, 0, 6, "layout3"); 

    textLabel2 = new QLabel(groupBox1, "textLabel2");
    layout3->addWidget(textLabel2);

    passesSpinBox = new QSpinBox(groupBox1, "passesSpinBox");
    passesSpinBox->setMaxValue(100);
    passesSpinBox->setMinValue(2);
    passesSpinBox->setValue(100);
    layout3->addWidget(passesSpinBox);
    QSpacerItem *spacer_2 = new QSpacerItem(191, 20, QSizePolicy::Expanding,
                                            QSizePolicy::Minimum);
    layout3->addItem(spacer_2);
    groupBox1Layout->addLayout(layout3);
    QSpacerItem *spacer_3 = new QSpacerItem(21, 75, QSizePolicy::Minimum,
                                            QSizePolicy::Expanding);
    groupBox1Layout->addItem(spacer_3);
    OptionsLayout->addWidget(groupBox1);

    layout6 = new QHBoxLayout(0, 0, 6, "layout6"); 
    QSpacerItem *spacer_4 = new QSpacerItem(131, 20, QSizePolicy::Expanding,
                                            QSizePolicy::Minimum);
    layout6->addItem(spacer_4);

    okButton = new QPushButton(this, "okButton");
    layout6->addWidget(okButton);

    cancelButton = new QPushButton(this, "cancelButton");
    layout6->addWidget(cancelButton);
    QSpacerItem *spacer_5 = new QSpacerItem(131, 20, QSizePolicy::Expanding,
                                            QSizePolicy::Minimum);
    layout6->addItem(spacer_5);
    OptionsLayout->addLayout(layout6);
    languageChange();

    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
    connect(fileBrowserButton, SIGNAL(clicked()), this, SLOT(getAsDirName()));
    connect(fileBrowserHelpButton, SIGNAL(clicked()), 
            this, SLOT(getHelpDirName()));

    // Set config_file to the right file
    config_file = getHomeDir() + CONFIG_FILE;

    // check if a file hasirc exists...
    std::ifstream in(config_file.c_str());
    if (in.is_open()) {
      // read it... this is really not the way it should be done...
      // TODO: create a config file format and parse it...
      std::string line;
      std::getline(in, line);
      if (in.bad()) goto initGUI;
      std::string asPathTmp(line);
      std::getline(in, line);
      if (in.bad()) goto initGUI;
      char *end = NULL;
      unsigned int listfileTmp = strtoul(line.c_str(), &end, 10);
      if (*end != '\0') {
        QMessageBox::warning(parent, tr("Error"), 
                             tr("Error reading options from hasirc\n"
                                "Using default values"),
                             QMessageBox::Ok, QMessageBox::NoButton);
      } else {
        std::getline(in, line);
        // TODO: get rid of that gotos :-)
        if (in.bad()) goto initGUI;
        unsigned int passesTmp = strtoul(line.c_str(), &end, 10);
        if (*end != '\0') {
          QMessageBox::warning(parent, tr("Error"), 
                               tr("Error reading options from hasirc\n"
                                  "Using default values"),
                               QMessageBox::Ok, QMessageBox::NoButton);
        } else {
          std::getline(in, line);
          if (in.bad()) goto initGUI;
          std::string helpPathTmp(line);

          asPath = asPathTmp;
          helpPath = helpPathTmp;
          listfile = (listfileTmp != 0);
          if ((passesTmp >= 2) && (passesTmp <= 100))
            // else we will use the default values...
            passes = passesTmp;
        }
      }
    }
  initGUI:
    pathnameLineEdit->setText(asPath.c_str());
    pathnameHelpLineEdit->setText(helpPath.c_str());
    listfileCheckBox->setChecked(listfile);
    passesSpinBox->setValue(passes);
  }

  // void Options::languageChange()
  // Sets the label texts for this widget.
  void Options::languageChange()
  {
    setCaption(tr("Options"));
    groupBox1->setTitle(tr("Assembler Options"));
    textLabel1->setText(tr("Path to the as binary:"));
    fileBrowserButton->setText(tr("Filebrowser"));
    textLabel3->setText(tr("Path to the help files:"));
    fileBrowserHelpButton->setText(tr("Filebrowser"));
    listfileCheckBox->setText(tr("Generate Listfile"));
    textLabel2->setText(tr("Max. Number of Passes:"));
    okButton->setText(tr("&OK"));
    cancelButton->setText(tr("&Cancel"));
  }

  // Options::~Options()
  // Destructor, saves the options to the config file
  Options::~Options()
  {
    saveOptions();
  }

  // void Options::saveOptions()
  // Writes the options to the file hasirc.
  void Options::saveOptions()
  {
    std::ofstream out(config_file.c_str(), std::ios::out | std::ios::trunc);
    if (out.is_open()) {
      out << asPath << std::endl;
      out << listfile << std::endl;
      out << passes << std::endl;
      out << helpPath << std::endl;
    }
  }

  // virtual void Options::accept()
  // Slot, gets called if the ok button is pressed.
  void Options::accept()
  {
    asPath = pathnameLineEdit->text().ascii();
    helpPath = pathnameHelpLineEdit->text().ascii();
    listfile = listfileCheckBox->isChecked();
    passes = passesSpinBox->value();
    QDialog::accept();
  }

  // virtual void Options::accept()
  // Slot, gets called if the cancel button is pressed or the dialog
  // is closed.
  void Options::reject()
  {
    pathnameLineEdit->setText(asPath.c_str());
    pathnameHelpLineEdit->setText(helpPath.c_str());
    listfileCheckBox->setChecked(listfile);
    passesSpinBox->setValue(passes);
    QDialog::reject();
  }

  // void Options::getAsDirName()
  // Sets the path to the directory where as is in with a dialog box.
  void Options::getAsDirName()
  {
    QString path = QFileDialog::getExistingDirectory
      (asPath.c_str(),  this,
       "get as directory",
       "Choose the path to the as binary",
       true);
    if (! path.isNull())
      pathnameLineEdit->setText(path);
  }

  // void Options::getHelpDirName()
  // Sets the path to the directory where the help files are 
  // located with a dialog box.
  void Options::getHelpDirName()
  {
    QString path = QFileDialog::getExistingDirectory
      (helpPath.c_str(),  this,
       "get help directory",
       "Choose the path to the help directory",
       true);
    if (! path.isNull())
      pathnameHelpLineEdit->setText(path);
  }

  // const std::string &Options::getHomeDir() [static]
  // Retrieves the users home directory. Not yet implemented for win32.
  const std::string &Options::getHomeDir()
  {
    static std::string home_dir;

#ifdef _WIN32
    // TODO: Make the windows code find the right directory
    home_dir = ".";
#else
    const char *env = getenv("HOME");
    if (env) {
      home_dir = env;
    } else {
      home_dir = ".";
    }
#endif

    return home_dir;
  }
}
