/* $Id: Options.hpp,v 1.2 2003/12/12 09:47:26 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Definition of a class holding the options to invoke the assembler.
 * The class is derived from QDialog and can show a dialog if necessary.
 */
#ifndef HASI_OPTIONS_HPP
#define HASI_OPTIONS_HPP

// system includes:
#include <string>
#include <fstream>
#include <qdialog.h>
#include <qvariant.h>

// forward Qt declarations to save compile time:
class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QCheckBox;
class QGroupBox;
class QLabel;
class QLineEdit;
class QPushButton;
class QSpinBox;

namespace GUI {

  /*
   * class Options
   */
  class Options : public QDialog
  {
    Q_OBJECT

  private:
    // assember options:
    std::string asPath; // path to the as binary
    std::string config_file; // The full path to the config file
    std::string helpPath;
    bool listfile;      // wether a listfile is written or not
    unsigned int passes;// the max. number of passes

    QGroupBox   *groupBox1;
    QLabel      *textLabel1;
    QLineEdit   *pathnameLineEdit;
    QPushButton *fileBrowserButton;
    QLabel      *textLabel3;
    QLineEdit   *pathnameHelpLineEdit;
    QPushButton *fileBrowserHelpButton;

    QCheckBox   *listfileCheckBox;
    QLabel      *textLabel2;
    QSpinBox    *passesSpinBox;
    QPushButton *okButton;
    QPushButton *cancelButton;

  protected:
    QVBoxLayout *OptionsLayout;
    QVBoxLayout *groupBox1Layout;
    QHBoxLayout *layout1;
    QHBoxLayout *layout2;
    QHBoxLayout *layout3;
    QHBoxLayout *layout4;
    QHBoxLayout *layout6;

  protected slots:
    virtual void languageChange();
    virtual void accept();
    virtual void reject();

  public:
    Options(QWidget *parent, const char *name);
    virtual ~Options();

    bool getListfile() const { return listfile; }
    const std::string &getAsPath() const { return asPath; }
    const std::string &getHelpPath() const { return helpPath; }
    unsigned int getPasses() const { return passes; }

    void saveOptions();

    static const std::string &getHomeDir();

  public slots:
    void getAsDirName();
    void getHelpDirName();

  };

}

#endif // HASI_OPTIONS_HPP
