/* $Id: QAssemblerOut.cpp,v 1.3 2003/12/12 09:47:26 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Implementation of a widget displaying the output of the assembler
 * for the user.
 */

// program includes:
#include "QAssemblerOut.hpp"

// system includes:
#include <qvariant.h>
#include <qpushbutton.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qsplitter.h>
#include <qwhatsthis.h>
#include <qregexp.h>

namespace GUI {

  // QAssemblerOutput::QAssemblerOutput(QWidget *parent, const char *name,
  //                                    bool modal, WFlags fl)
  //   : QDialog(parent, name, modal, fl)
  // Constructs the widgets in this dialog.
  QAssemblerOutput::QAssemblerOutput(QWidget *parent, const char *name,
                                     bool modal, WFlags fl)
    : QDialog(parent, name, modal, fl)
  {
    if (!name)
      setName("QAssemblerOutput");
    QAssemblerOutputLayout = new QVBoxLayout(this, 11, 6,
                                             "QAssemblerOutputLayout"); 

    splitter = new QSplitter(Qt::Vertical, this, "assembler output splitter");
    
    messagesTextEdit = new QTextEdit(splitter, "messagesTextEdit");
    messagesTextEdit->setVScrollBarMode(QTextEdit::AlwaysOn);
    messagesTextEdit->setHScrollBarMode(QTextEdit::AlwaysOn);
    messagesTextEdit->setWordWrap(QTextEdit::NoWrap);
    messagesTextEdit->setReadOnly(true);
    messagesTextEdit->setUndoRedoEnabled(false);
    messagesTextEdit->setTabChangesFocus(true);
    
    errorTextEdit = new QTextEdit(splitter, "errorTextEdit");
    errorTextEdit->setVScrollBarMode(QTextEdit::AlwaysOn);
    errorTextEdit->setHScrollBarMode(QTextEdit::AlwaysOn);
    errorTextEdit->setWordWrap(QTextEdit::NoWrap);
    errorTextEdit->setWrapPolicy(QTextEdit::AtWhiteSpace);
    errorTextEdit->setReadOnly(true);
    errorTextEdit->setUndoRedoEnabled(false);
    QAssemblerOutputLayout->addWidget(splitter);

    buttonLayout = new QHBoxLayout(0, 0, 6, "buttonLayout"); 

    dismissButton = new QPushButton(this, "dismissButton");
    buttonLayout->addWidget(dismissButton);
    QSpacerItem *spacer = new QSpacerItem(360, 20, QSizePolicy::Expanding,
                                          QSizePolicy::Minimum);
    buttonLayout->addItem(spacer);

    debuggerButton = new QPushButton(this, "debuggerButton");
    debuggerButton->setEnabled(false);
    buttonLayout->addWidget(debuggerButton);
    QAssemblerOutputLayout->addLayout(buttonLayout);
    languageChange();
    resize(QSize(620, 300).expandedTo(minimumSizeHint()));

    // signals and slots connections
    connect(dismissButton, SIGNAL(clicked()), this, SLOT(reject()));
    connect(debuggerButton, SIGNAL(clicked()), this, SLOT(accept()));
  }

  // QAssemblerOutput::~QAssemblerOutput()
  // Nothing to do here...
  QAssemblerOutput::~QAssemblerOutput()
  { }

  // void QAssemblerOutput::languageChange()
  // Sets the strings for the widgets.
  void QAssemblerOutput::languageChange()
  {
    setCaption(tr("Output"));
    QToolTip::add(messagesTextEdit,
                  tr("Shows the output of normal messages from the assembler."));
    QToolTip::add(errorTextEdit,
                  tr("Displays the error messages that occured during an assembly."));
    dismissButton->setText(tr("&Dismiss"));
    QToolTip::add(dismissButton, tr("Closes the window"));
    QWhatsThis::add(dismissButton, tr("Closes the window. \n"
                                      "Enabled after a complete assembly."));
    debuggerButton->setText(tr("&Go to Debugger"));
    QToolTip::add(debuggerButton,
                  tr("Closes the window and switches to the debugger."));
    QWhatsThis::add(debuggerButton,
                    tr("Closes the window and switches to the debugger.\n"
                       "The generated object file is loaded into the virtual memory."));
  }

  // void QAssemblerOutput::addLine(const std::string &msg)
  // Adds the line of text to the output. First a test is made if the
  // output is an error message, then it is put in the lower textbox, else
  // in the upper textbox.
  void QAssemblerOutput::addLine(const std::string &msg)
  {
    // if we get "Passes: xx" we can switch to the debugger...
    QRegExp passes("^Passes: \\d+(\n|\r\n)$");
    if (passes.search(msg.c_str()) != -1) {
      debuggerButton->setEnabled(true);
    }

    QRegExp error(".*error in line \\d+: .*$");
    if (error.search(msg.c_str()) == -1) {
      messagesTextEdit->append(msg.c_str());
    } else {
      errorTextEdit->append(msg.c_str());
    }

  }
}
