/* $Id: QDebugPane.hpp,v 1.8 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * The declaration of a Widget that contains the actual debugger. The CPU
 * information and the breakpoint list.
 */
#ifndef HASI_QDEBUGPANE_HPP
#define HASI_QDEBUGPANE_HPP

#include "Error.hpp" // workaround

// system includes:
#include <qwidget.h>
#include <list>

class QPushButton;
class QListBox;
class QListView;

// program includes:
#include "QHexLineEdit.hpp"
#include "CPU.hpp"

namespace GUI {

  /*
   * class QDebugPane
   */
  class QDebugPane : public QWidget
  {
    Q_OBJECT

  private:
    // the cpu we are controlling:
    Interpreter::CPU &cpu;

    // The Widgets that are shown inside this widget
    QPushButton  *stepButton;
    QPushButton  *nextButton;
    QPushButton  *runButton;
    QPushButton  *newBreakpoint;
    QPushButton  *deleteBreakpoint;
    QPushButton  *stopButton;
    QHexLineEdit *cpuAkku;
    QHexLineEdit *cpuSwap;
    QHexLineEdit *cpuIndex;
    QHexLineEdit *cpuSP;
    QHexLineEdit *cpuIP;
    QListView    *breakpointList;

    bool stop;

    void prepareRun(bool);

  public:
    // Con and Destructor:
    QDebugPane(Interpreter::CPU &cpu, 
               QWidget *parent, const char *name = NULL);
    virtual ~QDebugPane();

  signals:
    virtual void IPChanged(unsigned int address);
    virtual void SPChanged(unsigned int address);
    virtual void MemChanged(unsigned int address);
    virtual void Running(bool);

  public slots:
    void updateDisplay();
    virtual void changedAkku();
    virtual void changedHelp();
    virtual void changedIndex();
    virtual void changedSP();
    virtual void changedIP();
    
    virtual void breakpointSelected();

    void step();
    void next();
    void run();

    void stopPressed();

    void addBreakpoint();
    void remBreakpoint();
    
  };

}

#endif // HASI_QDEBUGPANE_HPP
