/* $Id: QDialogAddress.cpp,v 1.3 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// program includes:
#include "QDialogAddress.hpp"
#include "QHexIntValidator.hpp"

namespace GUI {

  // QDialogAddress::QDialogAddress(const unsigned int &address,
  //                                const std::string &comment,
  //                                QWidget *parent, const char *name,
  //                                bool modal, WFlags f)
  //   : QDialog(parent, name, modal, f)
  QDialogAddress::QDialogAddress(const unsigned int address,
                                 const std::string &comment,
                                 QWidget *parent, const char *name,
                                 bool modal, WFlags f)
    : QDialog(parent, name, modal, f)
  {
    topLayout = new QVBoxLayout(this);
    editGrid  = new QGridLayout(topLayout, 2, 2, 5);

    editComment = new QLineEdit(this);
    editComment->setMaxLength(100);
    editComment->setText(tr(comment.c_str()));
    editGrid->addWidget(editComment, 1, 1);
    editCommentLabel = new QLabel(editComment, tr("&Comment:"), this);
    editGrid->addWidget(editCommentLabel, 1, 0);
  
    editAddress = new QLineEdit(this);
    QHexIntValidator *validator = new QHexIntValidator(0, 65535, editAddress);
    editAddress->setValidator(validator);
    editAddress->setAlignment(Qt::AlignRight);
    QString rep;
    rep.sprintf("%04x", address);
    editAddress->setText(rep);
    editGrid->addWidget(editAddress, 0, 1);
    editAddressLabel = new QLabel(editAddress, tr("S&Address:"), this);
    editGrid->addWidget(editAddressLabel, 0, 0);

    topLayout->addStretch(100);

    buttonHBox = new QHBoxLayout(topLayout);

    buttonHBox->addStretch(100);

    okButton = new QPushButton(this);
    okButton->setText(tr("&OK"));
    buttonHBox->addWidget(okButton);
    cancelButton = new QPushButton(this);
    cancelButton->setText(tr("Cancel"));
    buttonHBox->addWidget(cancelButton);

    // Set the title
    setCaption(tr("Edit Address"));

    // Connect the Qt Signals to the corresponding slots
    connect(okButton, SIGNAL(pressed()), this, SLOT(okPressed()));
    connect(cancelButton, SIGNAL(pressed()), this, SLOT(reject()));
  }

  // QDialogAddress::~QDialogAddress()
  // Nothing to do here...
  QDialogAddress::~QDialogAddress()
  { }

  // void QDialogAddress::okPressed() , SLOT
  // Called if the ok button is pressed. Takes the input from the user
  // and puts it in the corresponding variables.
  void QDialogAddress::okPressed()
  {
    bool ok;
    QString addrstring = editAddress->text();
    address = addrstring.toUInt(&ok, 16);
    comment = editComment->text().ascii();
    emit(accept());
  }
}



