/* $Id: QHelpWindow.cpp,v 1.3 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,

 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * The implementation of the help window...
 * This is a simple html viewer for now. Will be improved when the 
 * documentation is finished...
 */

// system includes:
#include <qmainwindow.h>
#include <qtextbrowser.h>
#include <qstringlist.h>
#include <qpopupmenu.h>
#include <qmenubar.h>

// program includes:
#include "QHelpWindow.hpp"

namespace GUI {

  // QHelpWindow::QHelpWindow(const QString &home,  const QString &path, 
  //                          QWidget *parent, const char *name)
  // Constructor. Builds the window and shows the specified file.
  // All files must lay in the path given as the second arg.
  QHelpWindow::QHelpWindow(const QString &home,  const QString &path, 
                           QWidget *parent, const char *name)
    : QMainWindow(parent, name)
  {
    browser = new QTextBrowser(this);

    browser->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    connect(browser, SIGNAL(sourceChanged(const QString &)),
            this, SLOT(sourceChanged(const QString &)));

    setCentralWidget(browser);

    browser->mimeSourceFactory ()->addFilePath(path);    
    if (! home.isEmpty())
      browser->setSource(home);

    connect(browser, SIGNAL(highlighted(const QString &)),
            (QObject *)statusBar(), SLOT(message(const QString &)));

    QPopupMenu* go = new QPopupMenu( this );
    backwardId = go->insertItem(tr("&Backward"), browser, SLOT( backward()),
                                CTRL+Key_Left);
    forwardId = go->insertItem(tr("&Forward"), browser, SLOT(forward()),
                               CTRL+Key_Right);
    go->insertSeparator();
    go->insertItem(tr("&Close"), this, SLOT(close()),
                   CTRL+Key_X);
    menuBar()->insertItem(tr("&Go"), go);

    connect(browser, SIGNAL(backwardAvailable(bool)),
            this, SLOT(setBackwardAvailable(bool)));
    connect(browser, SIGNAL(forwardAvailable(bool)),
            this, SLOT(setForwardAvailable(bool)));

    menuBar()->setItemEnabled(backwardId, false);
    menuBar()->setItemEnabled(forwardId, false);

    resize(640, 700);
    browser->setFocus();
  }

  // QHelpWindow::~QHelpWindow()
  // Nothing to do here...
  // TODO: Send notification that the window was closed to the main program.
  QHelpWindow::~QHelpWindow()
  { }

  // void QHelpWindow::setBackwardAvailable(bool)
  // Slot, enables or disables the menu entry for backward navigation.
  void QHelpWindow::setBackwardAvailable(const bool b)
  {
    menuBar()->setItemEnabled(backwardId, b);
  }

  // void QHelpWindow::setForwardAvailable(bool)
  // Slot, enables or disables the menu entry for forward navigation.
  void QHelpWindow::setForwardAvailable(const bool b)
  {
    menuBar()->setItemEnabled(forwardId, b);
  }

  // void QHelpWindow::sourceChanged(const QString &)
  // Shows the name of the document in the title bar.
  void QHelpWindow::sourceChanged(const QString &url)
  {
    if (browser->documentTitle().isNull())
      setCaption("HASI Helpviewer - " + url);
    else
      setCaption("HASI Helpviewer - " + browser->documentTitle());
  }

  // void QHelpWindow::changeSource(const QString &)
  // Switches to a new document...
  void QHelpWindow::changeSource(const QString &home)
  {
    if (! home.isEmpty())
      browser->setSource(home);
  }
}

