/* $Id: QHexIntValidator.cpp,v 1.2 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,

 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Implementation of a Validator that accepts Hex values as input. The range
 * must include zero because the input handling is very strict.
 */

// program includes:
#include "QHexIntValidator.hpp"

namespace GUI {

  //  QHexIntValidator::QHexIntValidator(QObject *parent, const char *name)
  //    : QIntValidator(parent, name)
  // The constructor, nothing to be done here. All is handled by the
  // QIntValidator class.
  QHexIntValidator::QHexIntValidator(QObject *parent, const char *name)
    : QIntValidator(parent, name)
  { }

  // QHexIntValidator::QHexIntValidator(const int minimum, const int maximum,
  //                                    QObject *parent, const char *name)
  //  : QIntValidator(minimum, maximum, parent, name)
  // The constructor, nothing to be done here. All is handled by the
  // QIntValidator class. A range can be set.
  QHexIntValidator::QHexIntValidator(const int minimum, const int maximum,
                                     QObject *parent, const char *name)
    : QIntValidator(minimum, maximum, parent, name)
  { }

  // QHexIntValidator::~QHexIntValidator()
  // The destructor. Nothing to be done here.
  QHexIntValidator::~QHexIntValidator()
  { }

  // QValidator::State QHexIntValidator::validate(QString &, int &) const
  // Validates the input. The validation is done very strict. So intermediate
  // values as known from the Base class are not allowed.
  QValidator::State QHexIntValidator::validate(QString &input, int &pos) const
  {
    bool ok;

    // empty input defaults to 0
    if (input.isEmpty())
      return QValidator::Acceptable;

    // if there is a sole "-" and the buttom is below zero we can accept it
    if ((input == "-") && (bottom() < 0))
      return QValidator::Acceptable;

    // get the value of the input and check if it is a valid number
    // according to the Hex display
    int val = input.toInt(&ok, 16);

    // reject the input if it does not fit our restraints
    if ((! ok) || (val < bottom()) || (val > top()))
      return QValidator::Invalid;

    // ok, we can accept the input
    return QValidator::Acceptable;
  }

}
