/* $Id: QHexLineEdit.hpp,v 1.2 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * A LineEdit class accepting only Hex or Integer numbers. The acceptance can
 * be switched by a Button on the right of the widget. A button before the
 * LineEdit toggles a signed or unsigned display of the number.
 */
#ifndef HASI_QHEXLINEEDIT_HPP
#define HASI_QHEXLINEEDIT_HPP

// system includes:
#include <qwidget.h>
#include <qlineedit.h>
#include <qlayout.h>

// program includes:
#include "QHexIntValidator.hpp"
#include "QDecIntValidator.hpp"

class QPushButton;

namespace GUI {

  /*
   * class QHexLineEdit
   * The layout looks like the following ASCII Art:
   *
   * +---+----------------------+-------+
   * |u/s|                number|hex/dec|
   * +----------------------------------+
   *
   * u/s is a togglebutton that describes if it is an unsigned or signed value
   * number is a lineedit which holds the ASCII expression of the number
   * hex/dec is a togglebutton to decide if DECimal or HEXadecimal values
   * are allowed.
   */
  class QHexLineEdit : public QWidget
  {
    Q_OBJECT

  private:
    // Choice Between Hex and Int
    QPushButton       *signChoiceButton;
    QPushButton       *baseChoiceButton;
    QLineEdit         *numberLineEdit;
    QHexIntValidator  *numberHexValidator;
    QDecIntValidator  *numberDecValidator;
    QHBoxLayout       *boxLayout;

    // our state entries:
    bool sign;
    int base;
    int number;

  public:
    // unsigned hex is the standard...
    QHexLineEdit(QWidget *parent = NULL, const char *name = NULL);
    virtual ~QHexLineEdit();

    // we have to get and set the number
    int getNumber() const { return number; }
    void setNumber(int);

  public slots:
    virtual void toggleSignedUnsigned();
    virtual void toggleBase();
    virtual void changed(const QString &);
    virtual void retPressed();
  signals:
    // signal to simulate a QLineEdit Widget
    void returnPressed();
  };

}

#endif // HASI_QHEXLINEEDIT_HPP
