/* $Id: QMemoryPane.hpp,v 1.4 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * The declaration of a Widget that contains a MemoryTable, a chooser to set
 * the base of the MemoryTable and a HexLineEdit Widget to edit a selection.
 */
#ifndef HASI_QMEMORYPANE_HPP
#define HASI_QMEMORYPANE_HPP

// system includes:
#include <qwidget.h>

class QPushButton;
class QComboBox;
class QLineEdit;

// program includes:
#include "QMemoryTable.hpp"
#include "QHexLineEdit.hpp"
#include "MemoryChain.hpp"

namespace GUI {

  /*
   * class QMemoryPane
   */
  class QMemoryPane : public QWidget
  {
    Q_OBJECT

  private:
    // The Widgets that are shown inside this widget
    QMemoryTable *memTable;
    QPushButton *changeValueButton;
    QComboBox *memoryRangeComboBox;
    QHexLineEdit *memoryLineEdit;

  public:
    // Con and Destructor:
    QMemoryPane(QWidget *parent, const char *name = NULL);
    virtual ~QMemoryPane();

    // We have to set the MemoryChain used by this widget...
    void setMemory(Interpreter::MemoryChain *mem);

  signals:
    virtual void memoryChanged(unsigned int address);

  public slots:
    void comboChanged(int index);
    void memorySelected(int row, int col);
    void memoryEdited();
    void memoryWritten(unsigned int address);
  };

}

#endif // HASI_QMEMORYPANE_HPP
