/* $Id: QMemoryTable.hpp,v 1.3 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * The declaration of a fast and simple memory Tableview. This one displays
 * 16k words of memory in rows of 8 words. At the beginning of each row
 * the current address is printed, with a grey background. Every cell can
 * be selected, except the address cells. There is no possibility of selecting
 * more than one cell.
 */
#ifndef HASI_QMEMORYTABLE_HPP
#define HASI_QMEMORYTABLE_HPP

// system includes:
#include <qobject.h>
#include <qgridview.h>

class QPainter;

// program includes:
#include "MemoryChain.hpp"

namespace GUI {

  /*
   * class QMemoryTable
   * The following class implements a memory view with a fixed size
   * of 16KWords.
   * This implementation was done because the QTable class was far too slow
   * to allow custom headers on >2000 rows.
   */
  class QMemoryTable : public QGridView
  {
    Q_OBJECT

  private:
    // Our memory that is to be displayed. This can be NULL.
    Interpreter::MemoryChain *mem;
    // The beginning of the memory that is displayed.
    unsigned int base;
    // The cell width and height. Saved for speed.
    int pixelHeight, pixelWidth;
    // The selection. If both are -1 then nothing was selected. This could
    // be altered on derived classes (for example whole rows could be
    // selectable)
    int selectedRow, selectedCol;

  public:
    // Con and Destructor:
    QMemoryTable(Interpreter::MemoryChain *mem = NULL, QWidget *parent = NULL,
                 const char *name = NULL, WFlags fl = 0);
    ~QMemoryTable();
    
    // 
    void setMem(Interpreter::MemoryChain *m) { mem = m; repaintContents(); }
    Interpreter::MemoryChain *getMem() { return mem; }

    // Set the beginning of the memory displayed
    void setBase(const unsigned int b)
    {
      base = b;
      selectedRow = -1;
      selectedCol = -1;
      repaintContents();
    }
    // This has to be readable to outside classes too.
    unsigned int getBase() const { return base; }

    void changeSelection(const short newVal);

    // Get the selection-information
    const int getSelectedRow() const { return selectedRow; }
    const int getSelectedCol() const { return selectedCol; }

    void changedMem(unsigned int address);

  signals:
    // emited if the selection has changed
    virtual void select(int row, int col);

  protected:
    // This function has to be reimplemented to allow selection
    virtual void contentsMousePressEvent(QMouseEvent *e);
    // This function has to be reimplemented to allow a display *g
    virtual void paintCell(QPainter *p, int row, int col);
  };

}

#endif // HASI_QMEMORYTABLE_HPP
