/* $Id: main.cpp,v 1.3 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// program includes:
#include "MainForm.hpp"

#include "MemoryChain.hpp"
#include "RAM.hpp"
#include "IOTtyOut.hpp"
#include "CPU.hpp"

// system includes:
#include <qapplication.h>

int main(int argc, char **argv)
{
  QApplication app(argc, argv);
  Interpreter::MemoryChain memChain;
  Interpreter::RAM *mem = new Interpreter::RAM(65536);
  memChain.addMemorySegment(mem);
  Interpreter::IOTtyOut *outport = new Interpreter::IOTtyOut(0xc000);
  memChain.addMemorySegment(outport);
  Interpreter::Decoder decoder(memChain);
  Interpreter::CPU cpu(memChain, decoder);

  GUI::MainForm *m = new GUI::MainForm(cpu);
  m->resize(640, 480);
  m->show();
  app.connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));
  
  return app.exec();
}
