/* $Id: Error.cpp,v 1.4 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Implementation of some used exceptions.
 */

// system includes:
#include <iostream>

// program includes:
#include "Error.hpp"

namespace Error {

  // error::~error() throw ()
  error::~error() throw ()
  { }

  // illegal_instruction::illegal_instruction(const unsigned int)
  illegal_instruction::illegal_instruction(const unsigned int address)
    : address(address)
  { }

  // illegal_instruction::~illegal_instruction() throw ()
  illegal_instruction::~illegal_instruction() throw ()
  { }

  // division_by_zero::division_by_zero(const unsigned int)
  division_by_zero::division_by_zero(const unsigned int address)
    : address(address)
  { }

  // division_by_zero::~division_by_zero() throw ()
  division_by_zero::~division_by_zero() throw ()
  { }

  // file_error::file_error(const std::string &)
  file_error::file_error(const std::string &filename)
    : filename(filename)
  { }

  // file_error::~file_error() throw ()
  file_error::~file_error() throw ()
  { }

  // file_open_error::file_open_error(const std::string &) 
  file_open_error::file_open_error(const std::string &filename)
    : file_error(filename)
  { }

  // file_open_error::~file_open_error() throw ()
  file_open_error::~file_open_error() throw ()
  { }

  // void file_open_error::emit()
  // Prints an error message to stderr.
  void file_open_error::emit()
  {
    std::cerr << "Unable to open file: " << filename << std::endl;
  }

  // file_read_error::file_read_error(const std::string &)
  file_read_error::file_read_error(const std::string &filename)
    : file_error(filename)
  { }

  // file_read_error::~file_read_error() throw ()
  file_read_error::~file_read_error() throw ()
  { }

  // void file_read_error::emit()
  // Prints an error message to stderr.
  void file_read_error::emit()
  {
    std::cerr << "Error reading from file " << filename << "." << std::endl;
  }

}
