/* $Id: Error.hpp,v 1.5 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Declaration of some hardware-error conditions.
 */
#ifndef HASI_ERROR_HPP
#define HASI_ERROR_HPP

// system includes:
#include <exception>
#include <string>

namespace Error {
  
  /*
   * Base class for our error conditions 
   */
  class error : public std::exception
  {
  public:
    error() { };
    virtual ~error() throw ();
  };

  /* 
   * A illegal_instruction error
   */
  class illegal_instruction : public error
  {
  private:
    unsigned int         address;
  public:
    illegal_instruction(const unsigned int address);
    virtual ~illegal_instruction() throw ();

    unsigned int getAddress() const { return address; }
  };

  /* 
   * A division_by_zero error
   */
  class division_by_zero : public error
  {
  private:
    unsigned int         address;
  public:
    division_by_zero(const unsigned int address);
    virtual ~division_by_zero() throw ();

    unsigned int getAddress() const { return address; }
  };

  /*
   * Base class for file I/O errors
   */
  class file_error : public error
  {
  protected:
    std::string filename;
  public:
    file_error(const std::string &filename);
    virtual ~file_error() throw ();

    virtual void emit() = 0;
    const std::string &getFilename() const { return filename; }
  };

  /*
   * Error if a file could not be opened 
   */
  class file_open_error : public file_error
  {
  public:
    file_open_error(const std::string &filename);
    virtual ~file_open_error() throw ();
    
    virtual void emit();
  };

  /*
   * Error if a file could not be read 
   */
  class file_read_error : public file_error
  {
  public:
    file_read_error(const std::string &filename);
    virtual ~file_read_error() throw ();
    
    virtual void emit();
  };

}

#endif
