/* $Id: IOTtyOut.cpp,v 1.4 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Implementation of a memory based IO class. This one is derived from the
 * Memory class and implements writes to a tty.
 */

// system includes:
#include <iostream>

// program includes:
#include "IOTtyOut.hpp"

namespace Interpreter {

  // IOTtyOut::IOTtyOut(int)
  // Initializes the IOTtyOut. The content of the IOTtyOut is undefined.
  IOTtyOut::IOTtyOut(const unsigned int start)
    : Memory(1, start)
  { }

  // IOTtyOut::IOTtyOut(const IOTtyOut &)
  // Copies the memory. The content is copied 1:1. If there was no memory
  // in the argument new memory will be created.
  IOTtyOut::IOTtyOut(const IOTtyOut &m)
    : Memory(1, m.start)
  { }

  // IOTtyOut::~IOTtyOut()
  // Frees the IOTtyOut.
  IOTtyOut::~IOTtyOut()
  { }

  // bool IOTtyOut::write(const unsigned int, const short)
  // Writes a ASCII char to a tty. This is only done if the address is equal
  // to start. True or false will be returned according to the success of
  // the operation.
  bool IOTtyOut::write(const unsigned int addr, const short val)
  {
    if (addr == start) {
      std::cout << static_cast<char>((val & 0xff));
      return true;
    }
    return false;
  }

  // bool IOTtyOut::read(const unsigned int, short &)
  // False will be returned and the value is set to -1 (0xffff).
  bool IOTtyOut::read(const unsigned int addr, short &val) const
  {
    // hack to allow -W -Wall cleanly, hope compilers optimize it away
    val = static_cast<short>(addr);
    val = -1;
    return false;
  }

}
