/* $Id: Memory.cpp,v 1.3 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Implementation of the virtual base class Memory. Some functions
 * had to be implemented to allow a list of derived classes.
 * Memory normaly holds the memory itself and has size and start-address
 * information. This can be implemented differently for IO devices.
 */

// system includes:
#include <cstring>

// program includes:
#include "Memory.hpp"

namespace Interpreter {

  // Memory::Memory(int)
  // Initializes the Memory. The content of the RAM is undefined.
  Memory::Memory(const unsigned int size, const unsigned int start)
    : size(size), start(start)
  {
    //mem  = new short[size];
    mem = NULL;
  }

  // Memory::Memory(const Memory &)
  // Copies the memory. The content is copied 1:1.
  Memory::Memory(const Memory &m)
  {
    size = m.size;
    start = m.start;
    mem  = NULL;
  }

  // Memory::~Memory()
  // Frees the Memory. Virtual constructor, nothing to be done here as
  // the memory itself is not implemented.
  Memory::~Memory()
  { }

#if 0
  // bool Memory::write(const unsigned int addr, const short val)
  // Writes val to the specified address. False is returned if there was no
  // such address in this memory segment.
  // In the base implementation false is always returned...
  bool Memory::write(const unsigned int addr, const short val)
  {
    return false;
  }

  // bool Memory::read(const unsigned int addr, short &val) const
  // Reads a value from the specified address. False and -1 is returned
  // if there is no such address in this memory segment.
  // In the base implementation false is always returned.
  bool Memory::read(const unsigned int addr, short &val) const
  {
    val = -1;
    return false;
  }
#endif
}
