/* $Id: Memory.hpp,v 1.3 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * The (nearly) virtual base class Memory. Some functions had to be
 * implemented to allow a list of derived classes.
 * Memory normaly holds the memory itself and has size and start-address
 * information. This can be implemented differently for IO devices.
 */
#ifndef HASI_MEMORY_HPP
#define HASI_MEMORY_HPP

namespace Interpreter {

  /*
   * class Memory
   */
  class Memory
  {
  protected:
    // we have 16 bit per word, this will be allocated dynamically
    short *mem;
    // the size of the segment
    unsigned int size;
    // and the start of the mem we are using, this allows multiple
    // memory segments
    unsigned int start;

  public:
    // Con and Destructor:
    Memory(const unsigned int size, const unsigned int start = 0);
    Memory(const Memory &);
    virtual ~Memory();

    // The main functionality of memory, read and write to it.
    virtual bool write(const unsigned int addr, const short val) = 0;
    virtual bool read(const unsigned int addr, short &) const = 0;
    // The properties of this class. The size and the start must
    // not be altered after the initialization.
    virtual const unsigned int getStart() const { return start; }
    virtual const unsigned int getSize() const { return size; }
  };

}

#endif
