/* $Id: MemoryChain.cpp,v 1.2 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * The implementation of the MemoryChain class.
 */

// program includes:
#include "MemoryChain.hpp"

namespace Interpreter {

  // MemoryChain::MemoryChain()
  // Nothing to do here... 
  MemoryChain::MemoryChain()
  { }

  // MemoryChain::~MemoryChain()
  // Nothing to do here... the list will destroy itself.
  MemoryChain::~MemoryChain()
  { }

  // void MemoryChain::addMemorySegment(const Memory &)
  // Adds another chunk of memory to the chain. The memory is placed at
  // the beginning of the list so every new added addresses are queried
  // first on read and write operations.
  void MemoryChain::addMemorySegment(Memory *memseg)
  {
    memlist.push_front(memseg);
  }

  // bool MemoryChain::read(const unsigned int, short &)
  // Reads a word from memory. The chunks added to the list last are
  // read first. The read returns false if no memory was found at
  // the given address. In this case the value of val is unpredictable.
  // If true is returned val contains the value of the specified
  // address.
  bool MemoryChain::read(const unsigned int addr, short &val)
  {
    bool success = false;
    for (std::list<Memory *>::const_iterator it = memlist.begin();
         it != memlist.end(); it++) {
      if ((success = (*it)->read(addr, val)))
        break;
    }
    return success;
  }

  // bool MemoryChain::write(const unsigned int, const short)
  // Writes a word to memory. The chunks added to the list last are
  // tried first. The write is finished either if a valid writeable
  // chunk was found or the list was traversed and no write took
  // place. The return value is set according to the successful write.
  bool MemoryChain::write(const unsigned int addr, const short val)
  {
    bool success = false;
    for (std::list<Memory *>::iterator it = memlist.begin();
         it != memlist.end(); it++) {
      if ((success = (*it)->write(addr, val)))
        break;
    }
    return success;
  }

}
