/* $Id: MemoryChain.hpp,v 1.2 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * The declaration of a MemoryChain class. Chunks of Memory can be added
 * to the chain and are queried for write and read operations. The chunks
 * are queried in reverse order of their add.
 */
#ifndef HASI_MEMORYCHAIN_HPP
#define HASI_MEMORYCHAIN_HPP

// system includes:
#include <list>

// program includes:
#include "Memory.hpp"

namespace Interpreter {

  /*
   * class MemoryChain
   */
  class MemoryChain
  {
  private:
    // this list holds all chunks that were added to the chain:
    std::list<Memory *> memlist;

  public:
    // Con and Destructor:
    MemoryChain();
    virtual ~MemoryChain();

    // adds a chunk of memory to the chain, a copy of the segment is made
    void addMemorySegment(Memory *);
    // see Memory.hpp for the following functions:
    virtual bool read(const unsigned int addr, short &val);
    virtual bool write(const unsigned int addr, const short val);
  };

}
#endif
