/* $Id: ObjectFile.hpp,v 1.2 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Declaration of an class handling object files.
 */
#ifndef HASI_OBJECTFILE_HPP
#define HASI_OBJECTFILE_HPP

// system includes:
#include <string>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <list>

// program includes:
#include "MemoryChain.hpp"
#include "Error.hpp"

namespace Interpreter {

  /*
   * class ObjectFile
   */
  class ObjectFile
  {
  private:
    std::string filename;
    std::ifstream *file;

    std::list<std::string> stringparts;

    bool splitLine(const std::string &);
    void removeComment(std::string &);
    int hexStringToInt(const std::string &) throw (Error::file_read_error);
  public:
    // Con- and Destructor
    ObjectFile(const std::string &filename) throw (Error::file_open_error);
    ~ObjectFile();

    // The only function may be to read the file...
    void read(MemoryChain *) throw (Error::file_read_error);
  };

}

#endif
