/* $Id: RAM.cpp,v 1.3 2003/12/12 09:47:27 markus Exp $ */

/*
 * Copyright (c) 2003 Markus Hennecke
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the Author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Implementation of a Random Access Memory class. This one is derived from the
 * Memory class and implements read and write operations.
 */

// system includes:
#include <cstring>

// program includes:
#include "RAM.hpp"

namespace Interpreter {

  // RAM::RAM(int)
  // Initializes the RAM. The content of the RAM is undefined.
  RAM::RAM(const unsigned int size, const unsigned int start)
    : Memory(size, start)
  {
    mem = new short[size];
  }

  // RAM::RAM(const RAM &)
  // Copies the memory. The content is copied 1:1. If there was no memory
  // in the argument new memory will be created.
  RAM::RAM(const RAM &m)
    : Memory(m.size, m.start)
  {
    size = m.size;
    mem  = new short[size];
    if (m.mem)
      memcpy(mem, m.mem, size);
  }

  // RAM::~RAM()
  // Frees the RAM.
  RAM::~RAM()
  {
    if (mem)
      delete [] mem;
  }

  // bool RAM::write(const unsigned int, const short)
  // Writes a Word to RAM. This is only done if the address is within
  // range. True or false will be returned according to the success of
  // the operation.
  bool RAM::write(const unsigned int addr, const short val)
  {
    if (addr >= (size + start)) {
      return false;
    }
    mem[addr] = val;
    return true;
  }

  // bool RAM::read(const unsigned int, short &)
  // Reads a value from RAM and returns true if the address was in range.
  // If it is not in range false will be returned and the value is set to
  // -1 (0xffff).
  bool RAM::read(const unsigned int addr, short &val) const
  {
    if (addr >= (size + start)) {
      val = -1;
      return false;
    }
    val = mem[addr];
    return true;
  }

}
